#!/usr/bin/env python
# -*- coding: utf-8 -*-

from webmentiontools.urlinfo import UrlInfo
from webmentiontools.webmentionio import WebmentionIO

# If you have an access token from webmention.io,
# set it here. Some calls require it.

webmention_io_token = None 

wio = WebmentionIO(webmention_io_token)

# Get all links "mentioning" http://indiewebcamp.com/webmention
target_url = 'http://indiewebcamp.com/webmention'
ret = wio.linksToURL(target_url)

if not ret:
    print wio.error
else:
    for link in ret['links']:
        print 
        print 'Webmention.io ID: %s' % link['id']
        print '    Source: %s' % link['source']
        print '    Verification Date: %s' % link['verified_date']
        
        # Now use UrlInfo to get some more information about the source.
        # Most web apps showing webmentions, will probably do something 
        # like this.
        info = UrlInfo(link['source'])
        print '    Source URL info:'
        print '        Title: %s' % info.title()
        print '        Pub Date: %s' % info.pubDate()
        print '        in-reply-to: %s' % info.inReplyTo()
        print '        Author image: %s' % info.image()
        print '        Snippet: %s' % info.snippetWithLink(target_url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import urlparse
import requests
from bs4 import BeautifulSoup

class WebmentionSend():

    def __init__(self, source, target):
        self.source_url = source
        self.target_url = target
        self.receiver_endpoint = None

    def send(self, **kwargs):
        self.error = None
        self.requests_kwargs = kwargs
        r = self._discoverEndpoint()
        if not r:
            return False
        return self._notifyReceiver()

    def _discoverEndpoint(self):
        r = requests.get(self.target_url, **self.requests_kwargs)
        if r.status_code != 200:
            self.error = {
                'code':'BAD_TARGET_URL',
                'error_description': 'Unable to get target URL.',
                'request': 'GET %s' % self.target_url,
                'http_status': r.status_code,
            }
            return False
        html = r.text
        soup = BeautifulSoup(html)
        tag = soup.find('link', attrs={'rel': 'webmention'})
        if not tag:
            # backward compatibility with the webmention 0.1 spec
            tag = soup.find('link', attrs={'rel': 'http://webmention.org/'})

        if tag and tag['href']:
            # add the base scheme and host to relative endpoints
            self.receiver_endpoint = urlparse.urljoin(self.target_url, tag['href'])
            return True
        else:
            self.error = {
                'code': 'NO_ENDPOINT',
                'error_description': 'Unable to discover webmention endpoint.'
            }
            return False

    def _notifyReceiver(self):
        payload = {'source': self.source_url, 'target': self.target_url}
        headers = {'Accept': 'application/json'}
        r = requests.post(self.receiver_endpoint, data=payload, **self.requests_kwargs)

        request_str = 'POST %s (with source=%s, target=%s)' % (self.receiver_endpoint, self.source_url, self.target_url)
        if r.status_code / 100 != 2:
            self.error = {
                'code':'RECEIVER_ERROR',
                'request': request_str,
                'http_status': r.status_code,
                }
            try:
                self.error.update(r.json())
            except:
                self.error['body'] = r.text
            return False
        else:
            self.response = {
                'request': request_str,
                'http_status': r.status_code,
                'body': r.text
            }
            return True

if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
from bs4 import BeautifulSoup
from urlparse import urljoin

class UrlInfo():

    def __init__(self, url):
        self.url = url
        self.error = False
        self.fetchHTML()

    def fetchHTML(self):
        self.soup = None
        self.data = dict()
        self.data['links_to'] = []
        r = requests.get(self.url)
        if r.status_code != 200:
            self.error = True
            return
	# use apparent_encoding, seems to work better in the cases I tested.
    	r.encoding = r.apparent_encoding 
        self.soup = BeautifulSoup(r.text)

    def inReplyTo(self):
        if self.data.has_key('in_reply_to'):
            return self.data['in_reply_to']
        # Identify first class="u-in-reply-to" or rel="in-reply-to" link
        ir2 = self.soup.find('a', attrs={'class':'u-in-reply-to'})
        if not ir2:
            ir2 = self.soup.find('a', attrs={'rel':'in-reply-to'})
            if not ir2:
                return None
        ir2_link = ir2['href']
        self.data['in_reply_to'] = ir2_link
        return ir2_link

    def pubDate(self):
        # Get the time of the reply, if possible
        if self.data.has_key('pubDate'):
            return self.data['pubDate']

        ir2_time = self.soup.find(True, attrs={'class':'dt-published'})
        if ir2_time  and ir2_time.has_attr('datetime') :
            self.data['pubDate'] = ir2_time['datetime']
            return ir2_time['datetime']
        
    def title(self):
        if self.data.has_key('title'):
            return self.data['title']
        # Get the title
        title = self.soup.find('title').string
        self.data['title'] = title
        return title

    def image(self):
        if self.data.has_key('image'):
            return self.data['image']

        #Try using p-author
        author = self.soup.find(True, attrs={'class':'p-author'})
        if author:
            image = author.find('img')
            if image:
                image_src = image['src']
                self.data['image'] = urljoin(self.url, image_src)
                return self.data['image']

        # Try using h-card
        hcard = self.soup.find(True, attrs={'class':'h-card'})
        if hcard:
            image = hcard.find('img', attrs={'class':'u-photo'})
            if image:
                self.data['image'] = urljoin(self.url, image['src'])
                return self.data['image']

        # Last resort: try using rel="apple-touch-icon-precomposed"
        apple_icon = self.soup.find('link', attrs={'rel':'apple-touch-icon-precomposed'})
        if apple_icon:
            image = apple_icon['href']
            if image:
                self.data['image'] = urljoin(self.url, image)
                return self.data['image']

    def snippetWithLink(self, url):
        """ This method will try to return the first
        <p> or <div> that contains an <a> tag linking to
        the given URL.
        """
        link = self.soup.find("a", attrs={'href': url})
        if link:
            for p in link.parents:
                if p.name in ('p','div'):
                    return ' '.join(p.text.split()[0:30])
        return None


    def linksTo(self, url):
        # Check if page links to a specific URL.
        # please note that the test is done on the *exact* URL. If
        # you want to ignore ?parameters, please remove them in advance
        if url in self.data['links_to']:
            return True
        r = self.soup.find("a", attrs={'href': url})
        if r:
            self.data['links_to'].append(url)
            return True
        else:
            return False

if __name__ == '__main__':
   pass



from setuptools import setup, find_packages
from webmentiontools import __version__

setup(version=__version__,
      name="webmentiontools",
      author="Panayotis Vryonis",
      author_email="vrypan@gmail.com",
      description="Tools for webmention.org.",
      long_description=open('README.md').read(),
      packages=['webmentiontools'],
      install_requires=['beautifulsoup4', 'requests', 'docopt',],
      scripts=['bin/webmention-tools'],
      url='https://github.com/vrypan/webmention-tools',
      license='LICENSE',
      include_package_data=True,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
import json

class WebmentionIO():
    # Basic library for http://webmention.io/ API

    def __init__(self, access_token=None):
        self.access_token = access_token
        self.api_endpoint = 'http://webmention.io/api'

    def api_links_req(self, k, v):
        if k not in (None, 'target', 'domain'):
            return False
        url = "%s/links" % self.api_endpoint
        headers = {'Accept': 'application/json'}
        payload = { k:v, 'access_token': self.access_token }
        r = requests.get(url, headers=headers, params=payload)       
        if r.status_code != 200:
            self.error = r.text
            return False
        return json.loads(r.text)

    def linksToURL(self, url):
        links = self.api_links_req('target', url)
        if not links:
            return False
        else:
            return links

    def linksToDomain(self, domain):
        links = self.api_links_req('domain', domain)
        if not links:
            return False
        else:
            return links
        
    def linksToAll(self):
        pass

if __name__ == '__main__':
    pass
    """ Example:
    webmention_io_token = None # or set your token.
    wio = WebmentionIO(webmention_io_token)
    ret = wio.linksToURL('http://indiewebcamp.com/webmention')
    if not ret:
        print wio.error
    else:
        for l in ret['links']:
            print l['id'], l['source'], l['verified_date']
    """